//
//  CitiesTableViewController.swift
//  Capitales
//
//  Created by Santiago Pavón Gómez on 11/12/15.
//  Copyright © 2015 UPM. All rights reserved.
//

import UIKit
import MapKit

class CitiesTableViewController: UITableViewController {

    // URLs Base de la API y de los iconos.
    private let API_OPEN_WEATHER_URL = "http://api.openweathermap.org/data/2.5/weather"
    private let ICON_OPEN_WEATHER_URL = "http://openweathermap.org/img/w"
    
    // Hay que crearse una cuenta en http://api.openweathermap.org para poder acceder al API.
    private let APPID = ""
    
    // Array donde salvare las ciudades a mostrar
    private var cities: [String] = []
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        loadCities()
    }

    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        
        checkAPIID()
    }
    
    
    // Comprobar hemos puesto el valor de APIID
    private func checkAPIID() {
        if APPID == "" {
            let alert = UIAlertController(title: "Regístrese", message: "Para utilizar esta aplicación debe registrarse en http://openweathermap.org y obtener una clase de acceso al API", preferredStyle: .Alert)
            
            alert.addAction(UIAlertAction(
                title: "Vale",
                style: UIAlertActionStyle.Default) {(_: UIAlertAction) -> Void in
                    self.dismissViewControllerAnimated(true, completion: nil)
                })
            
            presentViewController(alert, animated: true, completion: nil)
        }
    }
    
    
    /** 
     * Leer el fichero con las ciudades a mostrar
     */
    private func loadCities() {
        
        print("Cargando Ciudades ...")
        
        let bundle = NSBundle.mainBundle()
        if let path = bundle.pathForResource("cities", ofType: "plist") {
            if let cities = NSArray(contentsOfFile: path) as? [String] {
                self.cities = cities
                tableView.reloadData()
            }
        }
    }
 
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    
    // MARK: - Table view data source

    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return cities.count
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("City Cell", forIndexPath: indexPath) as! CityTableViewCell

        let cityName = cities[indexPath.row]
        
        cell.cityLabel.text = cityName
        
        cell.mapView.alpha = 0
        
        configureCellForCityNamed(cityName, atIndexPath: indexPath)

        return cell
    }

    private func configureCellForCityNamed(name: String, atIndexPath ip: NSIndexPath) {
        
        var queue = dispatch_queue_create("cell queue", DISPATCH_QUEUE_SERIAL)
        dispatch_async(queue) {
            
            self.netActivityVisible = true
            defer {
                self.netActivityVisible = false
            }
            
            guard let query = "q=\(name)&units=metric&lang=sp&APPID=\(self.APPID)".stringByAddingPercentEncodingWithAllowedCharacters(.URLQueryAllowedCharacterSet()) else {
                print("Error escapando URL")
                return
            }
            
            guard let url = NSURL(string: "\(self.API_OPEN_WEATHER_URL)?\(query)") else {
                print("Error: URL mal formada")
                return
            }
            
            guard let data = NSData(contentsOfURL: url) else {
                print("Error: no se han podido descargar los datos.")
                return
            }
            
            do {
                if let dic = try NSJSONSerialization.JSONObjectWithData(data, options: NSJSONReadingOptions()) as? [String:AnyObject] {
                    
                    //print(dic)
                    
                    if let cod = dic["cod"] as? Int where cod == 200 {
                        self.updateCellAtIndexPath(ip, withDictionary: dic)
                    } else {
                        let msg = dic["message"] as? String ?? "Error: Desconocido"
                        print(msg)
                    }
                } else {
                    print("Los datos recibidos no tienen un formato inesperado.")
                
                }
            } catch let err  {
                print("No puedo sacar el JSON:", (err as NSError).localizedDescription)
                print(NSString(data: data, encoding: NSUTF8StringEncoding) ?? "Ni idea de lo que esta pasando.")
            }
        }
    }
    
    private func updateCellAtIndexPath(ip: NSIndexPath, withDictionary dic: [String:AnyObject]) {
        
        dispatch_async(dispatch_get_main_queue()) {
            
            if let cell = self.tableView.cellForRowAtIndexPath(ip) as? CityTableViewCell {
                
                if let cityName = dic["name"] as? String {
                    cell.cityLabel.text = cityName
                }
                
                let main = dic["main"] as! [String:AnyObject]
                let sys = dic["sys"] as! [String:AnyObject]
                let weather = dic["weather"] as! [[String:AnyObject]]
                let coord = dic["coord"] as! [String:AnyObject]
                
                if let latitude = coord["lat"] as? Float {
                    if let longitude = coord["lon"] as? Float {
                        
                        let center = CLLocationCoordinate2D(latitude: CLLocationDegrees(latitude), longitude: CLLocationDegrees(longitude))
                        let span = MKCoordinateSpanMake(0.3, 0.3)
                        let reg =  MKCoordinateRegionMake(center, span)
                        
                        cell.mapView.setRegion(reg, animated: false)
                        cell.mapView.mapType = .Hybrid
                        cell.mapView.userInteractionEnabled = false
                        
                        cell.mapView.alpha = 1
                    }
                }
                
                if let tempMin = main["temp_min"] as? Float {
                    cell.minTempLabel.text = "\(tempMin)ºC"
                }
                if let temp = main["temp"] as? Float {
                    cell.tempLabel.text = "\(temp)ºC"
                }
                if let tempMax = main["temp_max"] as? Float {
                    cell.maxTempLabel.text = "\(tempMax)ºC"
                }
                
                let dateFormatter = NSDateFormatter()
                dateFormatter.dateFormat = "HH:mm"
                
                if let ndate = sys["sunrise"] as? Double {
                    let date = NSDate(timeIntervalSince1970: ndate)
                    cell.sunriseLabel.text = dateFormatter.stringFromDate(date)
                }
                if let ndate = sys["sunset"] as? Double {
                    let date = NSDate(timeIntervalSince1970: ndate)
                    cell.sunsetLabel.text = dateFormatter.stringFromDate(date)
                }
                
                for var pos = 0 ; pos < min(3, weather.count) ; pos++ {
                    if let icon = weather[pos]["icon"] as? String {
                        self.setIcon(icon, forWeatherPosition: pos, atIndexPath: ip)
                    }
                }
            }
        }
    }
   
    /**
     Descargar un icono y ponerlo en la celda indicada, en una de las casillas reservadas para ello.
     */
    private func setIcon(iconName: String, forWeatherPosition wp: Int, atIndexPath ip: NSIndexPath) {
        
        let queue = dispatch_queue_create("icon queue", DISPATCH_QUEUE_SERIAL)
        dispatch_async(queue) {
            
            if let icon = self.getIconNamed(iconName) {
                
                dispatch_async(dispatch_get_main_queue()) {
                    
                    if let cell = self.tableView.cellForRowAtIndexPath(ip) as? CityTableViewCell {
                        
                        switch wp {
                        case 0:
                            cell.weather1ImageView.image = icon
                        case 1:
                            cell.weather0ImageView.image = icon
                        case 2:
                            cell.weather2ImageView.image = icon
                        default:
                            break
                        }
                    }
                }
            }
        }
    }
    
    /**
     Cache donde guardo los iconos que ya me he bajado.
     */
    private var iconsCache: [String:UIImage] = [:]
    
    /**
     Descargar un icono representando el tiempo
     */
    private func getIconNamed(name: String) -> UIImage? {
        
        if let img = iconsCache[name] { // Ya lo he bajado antes.
            return img
        }
        
        netActivityVisible = true
        defer {
            netActivityVisible = false
        }

        guard let url = NSURL(string: "\(ICON_OPEN_WEATHER_URL)/\(name).png") else {
            print("Error: URL mal formada")
            return nil
        }
        
        guard let data = NSData(contentsOfURL: url) else {
            print("Error: no se han podido descargar el icono.")
            return nil
        }
        
        guard let img = UIImage(data: data) else {
            print("Los datos recibidos no tienen un formato inesperado.")
            return nil
        }
        
        // Guardarlo en la cache
        iconsCache[name] = img
        
        return img
    }
    
    // MARK: - Network Activity
  
    /**
    Contador interno usado para saber cuantas veces hemos ejecutado:
    
    * UIApplication.sharedApplication().networkActivityIndicatorVisible = true
    
    Se activa el indicador de actividad en la red si este contador sea mayor que cero.
    
    Este contador no debe usarse directamente en el programa. Debe usarse la propiedad netActivityVisible.
    */
    private var netActivityCredit = 0 {
        didSet {
            UIApplication.sharedApplication().networkActivityIndicatorVisible = netActivityCredit > 0
        }
    }
    
    /**
     Activar o desactivar el indicador de actividad de red.
     
     Se ejecuta en el Main Thread.
     */
    private var netActivityVisible: Bool {
        get {
            var res = false
            dispatch_sync(dispatch_get_main_queue()) {
                res  = self.netActivityCredit > 0
            }
            return res
        }
        set {
            dispatch_sync(dispatch_get_main_queue()) {
                if (newValue) {
                    ++self.netActivityCredit
                } else {
                    --self.netActivityCredit
                }
            }
        }
    }
    
}
